@@ -28,18 +28,18 @@
 #raise NotImplementedError
 import logging
 import sys
+from datetime import datetime
 from StringIO import StringIO
 
 import numpy as np
 
-from mipp import CalibrationError
+from mipp import CalibrationError, ReaderError
 from mipp.xrit import bin_reader as rbin
 from mipp.xrit import Metadata, _xrit
 
 logger = logging.getLogger(__name__)
 
 
-
 __all__ = ['read_metadata']
 
 eval_np = eval
@@ -59,6 +59,8 @@ else:
                    'Performance will be slower.')
 
 
+MSG_HEIGHT = 35785831.00  # m
+
 # Reflectance factor for visible bands
 HRV_F = 25.15
 VIS006_F = 20.76
@@ -852,7 +854,10 @@ def read_metadata(prologue, image_files, epilogue):
     fp = StringIO(epilogue.data)
     ftr = read_epiheader(fp)
 
-    im = _xrit.read_imagedata(image_files[0])
+    try:
+        im = _xrit.read_imagedata(image_files[0])
+    except IndexError:
+        raise ReaderError("No image segments available")
 
     md = Metadata()
     md.calibrate = _Calibrator(
@@ -886,10 +891,13 @@ def read_metadata(prologue, image_files, epilogue):
              ftr["UpperEastColumnActual"],
              ftr["UpperWestColumnActual"]]])
 
-        md.coff = (ftr["Lower" + ew_.capitalize() + "ColumnActual"]
-                   + im.navigation.coff + 1)
+        md.coff = (ftr["Lower" + ew_.capitalize() + "ColumnActual"] - 1
+                   + im.navigation.coff - 1)
         md.loff = im.navigation.loff + \
-            segment_size * (im.segment.seg_no - 1) + 2
+            segment_size * (im.segment.seg_no - 1) - 1
+        if (hdr["GeometricProcessing"]["EarthModel"]["TypeOfEarthModel"] < 2):
+            md.coff += 1.5
+            md.loff += 1.5
 
     else:
         md.first_pixel = hdr["ReferenceGridVIS_IR"]["GridOrigin"]
@@ -900,8 +908,16 @@ def read_metadata(prologue, image_files, epilogue):
             ftr["EasternColumnActual"],
             ftr["WesternColumnActual"]]])
 
-        md.coff = im.navigation.coff
-        md.loff = im.navigation.loff + segment_size * (im.segment.seg_no - 1)
+        md.coff = im.navigation.coff - 1
+        md.loff = im.navigation.loff + \
+            segment_size * (im.segment.seg_no - 1) - 1
+
+        if (hdr["GeometricProcessing"]["EarthModel"]["TypeOfEarthModel"] < 2):
+            md.coff += .5
+            md.loff += .5
+
+    md.x_pixel_size = np.deg2rad(2.**16 / im.navigation.cfac) * MSG_HEIGHT * -1
+    md.y_pixel_size = np.deg2rad(2.**16 / im.navigation.lfac) * MSG_HEIGHT * -1
 
     md.data_type = im.structure.nb
     md.no_data_value = no_data_value
