@@ -1,8 +1,11 @@
 import sys
 import os
-from distutils.util import get_platform
 
-proj_path = os.path.abspath(os.path.join(os.path.dirname(__file__),'..'))
+home = os.path.abspath(os.path.join(os.path.dirname(__file__),'..'))
+sys.path.insert(0, home)
+
+"""
+from distutils.util import get_platform
 build_base = 'build'
 plat = get_platform()
 
@@ -10,10 +13,10 @@ python_version = '.'.join([str(i) for i in sys.version_info[:2]])
 build_purelib = os.path.join(build_base, 'lib' + '-' + python_version) # is it at all used ?
 build_platlib = os.path.join(build_base, 'lib.' + plat + '-' + python_version)
 
-build_path = os.path.join(proj_path, build_platlib)
+build_path = os.path.join(home, build_platlib)
 if not os.path.isdir(build_path):
     assert False, "No such build path '%s'"%build_path
 sys.path.insert(0, build_path)
-
 if __name__ == '__main__':
-    print build_platlib
+    print build_path
+"""
